// prettyprint.[ch]
// print non-trivial data to screen
//
// (c) 2007, Willem de Bruijn, Vrije Universiteit Amsterdam
// email at willem -_at_- computer.org
//
// LGPL license applies

#ifdef __KERNEL__
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/if_ether.h>	
#include <linux/in.h>	
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#else
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#if linux
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#else
#include "../hw/proto.h"
#endif
#endif

#include "prettyprint.h"
#include "macros.h"

/** Pretty print data in vertically split [hex | decimal] notation 
 *
 *  The passed string must be at least 80 bytes.
 * */
int
writedata(char *out, int olen, const char *data, int dlen) {
    	int i = 0, off = 0;
	int elem;

	olen--; // leave room for the terminating '\0'
	do {
		// phase 1: print hex
		for (elem = 0; elem < HEXWIDTH && i + elem < dlen; elem++) {
			off += snprintf(out + off, olen - off, "%x%x ", 
				        (data[i + elem] & 0xf0) >> 4,
				         data[i + elem] & 0xf);
			
			if (elem == (HEXWIDTH / 2) - 1) {
				out[off] = ' ';
				off++;
			}
		}
		
		// fill out the last line
		for (; elem < HEXWIDTH; elem ++) {
			out[off] = ' ';
			out[off + 1] = ' ';
			out[off + 2] = ' ';
			off += 3;
		}
		
		// insert room
		out[off] = ' ';
		out[off + 1] = ' ';
		out[off + 2] = ' ';
		off += 3;

		// phase 2: print visible
		for (elem = 0; elem < HEXWIDTH && i + elem < dlen; elem++) {
			if (data[i + elem] >= 32 && data[i + elem] < 126)
				out[off + elem] = data[i + elem];
			else
				out[off + elem] = '.';
		}
		off += elem;
		out[off] = '\n';
		off++;
		i += HEXWIDTH;
	} while (i < dlen && off < olen);

	out[off] = '\n';
	off++;
	out[off] = '\0';
	return off;
}

void
displaydata(const char *data, int dlen)
{
	char *out;
	int len, mlen;
#ifndef __KERNEL__
	int ret;
#endif
	
	if (dlen) {
		len = 5 * dlen;
		mlen = max(len + 1, 800);

		// allocate the block. difficult only because of 
		// possible execution in kernel interrupt context.
#ifdef __KERNEL__
		if (in_interrupt()) {
			out = kmalloc(mlen, GFP_ATOMIC);
			memset(out, 0, mlen);
		}
		else
#endif		
			out = mycalloc(mlen, 1);

		// malloc failed error handling
		if (!out) {
			const char error[] = "BUG in displaydata\n";
#ifdef __KERNEL__
			printk(error);
#else
			ret = write(1, error, 20);
#endif
			return;
		}

		// fill in contents and write output
		len = writedata(out, len - 1, data, dlen);
		out[len] = '\0';
#ifdef __KERNEL__
		printk("%s", out);
#else
		ret = write(1, out, len + 1);
#endif
		myfree(out);
	}
}

EXPORT_SYMBOL(displaydata);

